import {Component, ElementRef, Input, OnInit, ViewChild} from '@angular/core';
import {Observable} from 'rxjs';
import {ActivitiesService} from './activities.service';
import {animate, style, transition, trigger} from '@angular/animations';
import {ActivityModalComponent} from './activity-modal/activity-modal.component';
import {Activity, DataCard} from '@nspop/gm-web-facade-api';
import {NoteModalComponent} from './note-modal/note-modal.component';
import {CarePlanService} from '../api/care-plan.service';

@Component({
  selector: 'app-activities',
  templateUrl: './activities.component.html',
  styleUrls: ['./activities.component.scss'],
  animations: [
    trigger(
      'inOutAnimation',
      [
        transition(
          ':enter',
          [
            style({ opacity: 0 }),
            animate('0.5s ease-out',
              style({opacity: 1 }))
          ]
        ),
        transition(
          ':leave',
          [
            style({ opacity: 1 })
          ]
        )
      ]
    )
  ]
})
export class ActivitiesComponent implements OnInit{

  public pastActivities$: Observable<Activity[]> =  this.activitiesService.pastActivities$;
  public futureActivities$: Observable<Activity[]> =  this.activitiesService.futureActivities$;

  public activeDataCard$: Observable<DataCard | undefined> =  this.activitiesService.activeDataCard$;

  public dividerPassed = false;

  @ViewChild('divider') public divider: ElementRef | undefined;
  @ViewChild('button') public button: HTMLButtonElement | undefined;

  @ViewChild('activityModal') public activityModal: ActivityModalComponent | undefined;
  @ViewChild('noteModal') public noteModal: NoteModalComponent | undefined;

  public editMeasurementsDialog: {
    isOpen: boolean;
    observationType: 'regular' | 'ultrasound';
    allowTypeChange: boolean;
    activity?: Activity;
    allowActivityChange: boolean;
  } = {isOpen: false, observationType: 'regular', allowTypeChange: true, allowActivityChange: false};
  public saveListeners: (() => any)[] = [];
  public disableSaveButton = false;
  public viewActivityDialog: {isOpen: boolean; activityId?: string} = {isOpen: false};
  public dataCard: DataCard | undefined;

  constructor(
    private activitiesService: ActivitiesService,
    private carePlanService: CarePlanService,
    ) {
  }

  private saveCallback = () => this.carePlanService.refresh();

  ngOnInit(): void {
    this.activitiesService.openActivityModalEvent$.subscribe((activity: Activity) => {
      this.activityModal?.setActivity(activity);
      this.activityModal?.setVisible(true);
    });

    this.activitiesService.openNoteModalEvent$.subscribe((activity: Activity) => {
      this.noteModal?.setActivity(activity);
      this.noteModal?.setVisible(true);
    });

    this.activitiesService.openMeasurementsModalEvent$.subscribe((activity: Activity) => {
      this.createMeasurements(activity);
    });

    this.saveListeners.push(this.saveCallback);
    this.activeDataCard$.subscribe((dataCard?: DataCard) => {
      this.dataCard = dataCard;
    });
  }

  public addActivity(): void {
    this.activitiesService.emitOpenActivityModalEvent(undefined);
  }

  public createMeasurements(activity: Activity): void {
    if (this.editMeasurementsDialog) {
      this.editMeasurementsDialog.observationType = 'regular';
      this.editMeasurementsDialog.isOpen = true;
      this.editMeasurementsDialog.allowTypeChange = true;
      this.editMeasurementsDialog.activity = activity;
      this.editMeasurementsDialog.allowActivityChange = false;
    }
  }

  public onSaveMeasurements(): void {
    console.log('saving measurements...');
  }

  public onScroll($event: Event): void {
    if (this.divider) {
      const container = $event.target as HTMLDivElement;
      const elementPosition = this.divider.nativeElement.offsetTop;
      const scrollPosition = container.scrollTop + container.offsetTop;
      this.dividerPassed = scrollPosition >= elementPosition;
    }
  }
}
