import {ComponentFixture, fakeAsync, flush, TestBed} from '@angular/core/testing';

import { ActivityModalComponent } from './activity-modal.component';
import {FormGroup, FormGroupDirective, FormsModule, ReactiveFormsModule} from '@angular/forms';
import {AngularMaterialModule} from '../../modules/angular-material.module';
import {BrowserAnimationsModule} from '@angular/platform-browser/animations';
import {Component, Input} from '@angular/core';
import {Activity, CarePlan} from '@nspop/gm-web-facade-api';
import {Subject, throwError} from 'rxjs';
import {CarePlanService} from '../../api/care-plan.service';
import {ActivitiesService} from '../activities.service';

describe('ActivityModalComponent', () => {
  let component: ActivityModalComponent;
  let fixture: ComponentFixture<ActivityModalComponent>;

  let activitiesServiceStub: ActivitiesService;
  let carePlanSubjectStub: Subject<CarePlan>;
  let carePlanServiceStub: CarePlanService;

  @Component({selector: 'app-modal', template: '<div><ng-content></ng-content></div>'})
  class ModalComponent {
    @Input() public header = '';
    private isModalVisible = false;
    public setVisible(isVisible: boolean): void {
      this.isModalVisible = isVisible;
    }
  }
  beforeEach(async () => {
    carePlanSubjectStub = new Subject<CarePlan>();
    carePlanServiceStub = {
      carePlan$: carePlanSubjectStub.asObservable(),
      update: () => carePlanSubjectStub.asObservable(),
      refresh: () => {},
    } as any;

    activitiesServiceStub = {
      carePlan: { activities: []},
      dataCard: { id: '123'},
    } as any;

    await TestBed.configureTestingModule({
      declarations: [ ActivityModalComponent, ModalComponent],
      imports: [
        ReactiveFormsModule,
        FormsModule,
        ReactiveFormsModule,
        AngularMaterialModule,
        BrowserAnimationsModule,
      ],
      providers: [
        {provide: CarePlanService, useValue: carePlanServiceStub},
        {provide: ActivitiesService, useValue: activitiesServiceStub},
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(ActivityModalComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('when creating a new activity', () => {
    it('closes modal if submitting carePlan is a success', () => {
      component.activity = undefined;

      component.setVisible(true);
      component.carePlan = {id: '123', version: 456, activities: []} as any as CarePlan;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      carePlanSubjectStub.next();
      expect(component.isVisible()).toBe(false);
    });

    it('does not close modal if submitting carePlan fails', () => {
      spyOn(component.carePlanService, 'update').and.returnValue(throwError('error'));
      component.activity = undefined;

      component.setVisible(true);
      component.carePlan = {id: '123', version: 456, activities: []} as any as CarePlan;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      carePlanSubjectStub.next();
      expect(component.isVisible()).toBe(true);
    });
  });

  describe('when editing an activity', () => {
    it('closes modal if submitting carePlan is a success', () => {
      const activity = { id: '1', status: 'expected'} as any as Activity;
      component.activity = activity;

      component.setVisible(true);
      component.carePlan = {id: '123', version: 456, activities: [ activity ]} as any as CarePlan;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      carePlanSubjectStub.next();
      expect(component.isVisible()).toBe(false);
    });

    it('does not close modal if submitting carePlan fails', () => {
      spyOn(component.carePlanService, 'update').and.returnValue(throwError('error'));
      const activity = { id: '1', status: 'expected'} as any as Activity;
      component.activity = activity;

      component.setVisible(true);
      component.carePlan = {id: '123', version: 456, activities: [ activity ]} as any as CarePlan;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      carePlanSubjectStub.next();
      expect(component.isVisible()).toBe(true);
    });

    it('does not submit if there is no carePlan', () => {
      spyOn(component.carePlanService, 'update');
      const activity = { id: '1', status: 'expected'} as any as Activity;
      component.activity = activity;

      component.setVisible(true);
      component.carePlan = undefined;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      expect(component.carePlanService.update).toHaveBeenCalledTimes(0);
    });

    it('does not submit if there is no carePlan', () => {
      spyOn(component.carePlanService, 'update');
      const activity = { id: '1', status: 'expected'} as any as Activity;
      component.activity = activity;

      component.setVisible(true);
      component.carePlan = {id: '123', version: 456, activities: [ activity ]} as any as CarePlan;
      component.dataCard = undefined;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      expect(component.carePlanService.update).toHaveBeenCalledTimes(0);
    });

    it('does not submit if it cannot find the activity', () => {
      spyOn(component.carePlanService, 'update');
      const activity = { id: '1', status: 'expected'} as any as Activity;
      component.activity = activity;

      component.setVisible(true);
      component.carePlan = {id: '123', version: 456, activities: [ { id: '2', status: 'expected'} ]} as any as CarePlan;
      component.form?.setValue(
        {
          title: 'Ultralydscanning',
          description: 'vi scanner og alt det der',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      expect(component.carePlanService.update).toHaveBeenCalledTimes(0);
    });

    it('does not submit if the form is invalid', () => {
      spyOn(component.carePlanService, 'update');
      const activity = { id: '1', status: 'expected'} as any as Activity;
      component.activity = activity;

      component.setVisible(true);
      component.carePlan = {id: '123', version: 456, activities: [ { id: '2', status: 'expected'} ]} as any as CarePlan;
      component.form?.setValue(
        {
          title: '',
          description: '',
          status: 'expected',
          fromWeek: new Date(),
          toWeek: new Date(),
          note: {content: 'det gik nogenlunde'}
        });

      component.submit(component.form, component.formDirective);
      expect(component.carePlanService.update).toHaveBeenCalledTimes(0);
    });
  });

  it('sets editMode = true if there is an activity, when setting Activity', () => {
    expect(component.isEditMode).toBeFalsy();

    const activity = {status: 'expected', id: '1'} as Activity;
    component.setActivity(activity);

    expect(component.isEditMode).toBe(true);
  });

  it('sets editMode = false if there is no activity, when setting Activity', () => {
    const activity = {status: 'expected', id: '1'} as Activity;
    component.setActivity(activity);
    expect(component.isEditMode).toBe(true);

    component.setActivity(undefined);
    expect(component.isEditMode).toBe(false);
  });
});
