import { ComponentFixture, TestBed } from '@angular/core/testing';

import { EditPlanModalComponent } from './edit-plan-modal.component';
import {WeekNumberPipe} from '../../pipes/week-number.pipe';
import {MatButtonModule} from '@angular/material/button';
import {ModalComponent} from '../../modals/modal/modal.component';
import {Component, Input} from '@angular/core';
import {MatIcon, MatIconModule} from '@angular/material/icon';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';
import {MatCheckboxModule} from '@angular/material/checkbox';
import {DirtyGuard} from '../../confirm-navigation-if-dirty.framework';
import {ActivitiesService} from '../activities.service';
import {Subject, throwError} from 'rxjs';
import {CarePlan} from '@nspop/gm-web-facade-api';
import {CarePlanService} from '../../api/care-plan.service';

@Component({
  // tslint:disable-next-line:component-selector
  selector: 'mat-icon',
  template: '<span></span>'
})
class MockMatIconComponent {
  @Input() svgIcon: any;
  @Input() fontSet: any;
  @Input() fontIcon: any;
}

describe('EditPlanModalComponent', () => {
  let component: EditPlanModalComponent;
  let fixture: ComponentFixture<EditPlanModalComponent>;

  let activitiesServiceStub: ActivitiesService;
  let carePlanSubjectStub: Subject<CarePlan>;
  let carePlanServiceStub: CarePlanService;

  beforeEach(async () => {
    activitiesServiceStub = {
      carePlan: {
        id: '123',
        version: 456,
        activities: [
          {id: '1', status: 'finished', ownership: 'currentUser'},
          {id: '2', status: 'planned', ownership: 'otherUser'},
          {id: '3', status: 'expected', ownership: 'currentUser'},
          {id: '4', status: 'planned', ownership: 'currentUser'},
          {id: '5', status: 'cancelled', ownership: 'currentUser'},
        ]
      } as any as CarePlan,
    } as any;

    carePlanSubjectStub = new Subject<CarePlan>();

    carePlanServiceStub = {
      carePlan$: carePlanSubjectStub.asObservable(),
      update: () => carePlanSubjectStub.asObservable(),
      refresh: () => {
      },
    } as any;

    await TestBed.configureTestingModule({
      declarations: [ EditPlanModalComponent, WeekNumberPipe, ModalComponent, WeekNumberPipe],
      imports: [MatIconModule, MatButtonModule, ReactiveFormsModule, FormsModule, MatCheckboxModule],
      providers: [
        {provide: DirtyGuard, useValue: {canDeactivate: () => true}},
        {provide: ActivitiesService, useValue: activitiesServiceStub},
        {provide: CarePlanService, useValue: carePlanServiceStub},
      ]
    })
    .overrideModule(MatIconModule, {
      remove: {
        declarations: [MatIcon],
        exports: [MatIcon]
      },
      add: {
        declarations: [MockMatIconComponent],
        exports: [MockMatIconComponent]
      }
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(EditPlanModalComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should return true if some but not all activities are selected', () => {
    component.editableActivities = [0, 1];
    expect(component.someButNotAllSelected()).toBeFalse();
    component.selectedActivities[0] = false;
    component.selectedActivities[1] = true;
    expect(component.someButNotAllSelected()).toBeTrue();
  });

  it('should return false if all activities are selected', () => {
    component.editableActivities = [0, 1];
    component.selectedActivities[0] = false;
    component.selectedActivities[1] = true;
    expect(component.someButNotAllSelected()).toBeTrue();
    component.selectedActivities = component.selectedActivities.map((value) => true);
    expect(component.someButNotAllSelected()).toBeFalse();
  });

  it('should only apply to editable activities', () => {
    component.editableActivities = [1, 2];
    component.selectedActivities = [false, true, true];
    expect(component.allSelected()).toBe(true);
    expect(component.someButNotAllSelected()).toBe(false);

    component.selectedActivities = [false, false, true];
    expect(component.allSelected()).toBe(false);
    expect(component.someButNotAllSelected()).toBe(true);

    component.selectedActivities = [false, false, false];
    expect(component.allSelected()).toBe(false);
    expect(component.someButNotAllSelected()).toBe(false);
  });

  it('should toggle all editable activities', () => {
    component.editableActivities = [0, 1, 2];
    component.selectedActivities = [false, false, false];
    expect(component.allSelected()).toBe(false);
    component.toggleAll();
    expect(component.allSelected()).toBe(true);
    component.toggleAll();
    expect(component.allSelected()).toBe(false);
  });

  it('should set all to true if only some editable activities are selected', () => {
    component.editableActivities = [0, 1, 2];
    component.selectedActivities = [true, false, true];
    expect(component.allSelected()).toBe(false);
    component.toggleAll();
    expect(component.allSelected()).toBe(true);
  });

  it('toggles only editable activities', () => {
    component.editableActivities = [1, 2];
    component.selectedActivities = [false, false, false];
    component.toggleAll();
    expect(component.selectedActivities).toEqual([false, true, true]);
    component.toggleAll();
    expect(component.selectedActivities).toEqual([false, false, false]);
  });

  it('populates editableActivities only with activities that are not finished or from other users, when set visible', () => {
    component.editableActivities = [];
    expect(component.editableActivities).toEqual([]);
    component.setVisible(true);
    expect(component.editableActivities).toEqual([2, 3, 4]);
  });

  it('populates selectedActivities with all activities that are not cancelled in the careplan with value true, false otherwise', () => {
    component.selectedActivities = [];
    expect(component.selectedActivities).toEqual([]);
    component.setVisible(true);
    expect(component.selectedActivities).toEqual([true, true, true, true, false]);
  });

  it('closes modal if submitting carePlan is a success', () => {
    component.setVisible(true);
    component.carePlan = {
      id: '123',
      version: 456,
      activities: [ {status: 'expected', id: '1', ownership: 'currentUser'} ]
    } as any as CarePlan;

    component.editableActivities = [0];
    component.selectedActivities = [true];

    component.submit();
    carePlanSubjectStub.next();
    expect(component.isVisible()).toBe(false);
  });

  it('does not close modal if submitting carePlan fails', () => {
    spyOn(component.carePlanService, 'update').and.returnValue(throwError('error'));
    component.setVisible(true);
    component.carePlan = {
      id: '123',
      version: 456,
      activities: [ {status: 'expected', id: '1', ownership: 'currentUser'} ]
    } as any as CarePlan;

    component.editableActivities = [0];
    component.selectedActivities = [true];

    component.submit();
    expect(component.isVisible()).toBe(true);
  });

  it('does not submit if there is no careplan set', () => {
    spyOn(component.carePlanService, 'update');
    component.setVisible(true);
    component.carePlan = undefined;
    component.editableActivities = [];
    component.selectedActivities = [];

    component.submit();
    expect(component.carePlanService.update).toHaveBeenCalledTimes(0);
  });

  it('sets activity.status = cancelled if not selected and allowed to change', () => {
    spyOn(component.carePlanService, 'update').and.returnValue(throwError('error'));
    component.setVisible(true);
    const activity = {status: 'expected', id: '1', ownership: 'currentUser'};
    component.carePlan = {
      id: '123',
      version: 456,
      activities: [ activity ]
    } as any as CarePlan;

    component.editableActivities = [0]; // allowed to change first activity
    component.selectedActivities = [false]; // not selected

    expect(activity.status).toBe('expected');
    component.submit();
    expect(activity.status).toBe('cancelled');
  });

  it('sets activity.status = expected if selected, allowed to change, and was cancelled before', () => {
    spyOn(component.carePlanService, 'update').and.returnValue(throwError('error'));
    component.setVisible(true);
    const activity = {status: 'cancelled', id: '1', ownership: 'currentUser'};
    component.carePlan = {
      id: '123',
      version: 456,
      activities: [ activity ]
    } as any as CarePlan;

    component.editableActivities = [0]; // allowed to change first activity
    component.selectedActivities = [true]; // selected

    expect(activity.status).toBe('cancelled');
    component.submit();
    expect(activity.status).toBe('expected');
  });

  it('it does not change activity.status if activity is selected and was not cancelled before', () => {
    spyOn(component.carePlanService, 'update').and.returnValue(throwError('error'));
    component.setVisible(true);
    const activity = {status: 'planned', id: '1', ownership: 'currentUser'};
    component.carePlan = {
      id: '123',
      version: 456,
      activities: [ activity ]
    } as any as CarePlan;

    component.editableActivities = [0]; // allowed to change first activity
    component.selectedActivities = [true]; // selected

    expect(activity.status).toBe('planned');
    component.submit();
    expect(activity.status).toBe('planned');
  });

  it('it does not change activity.status if not allowed to change activity', () => {
    spyOn(component.carePlanService, 'update').and.returnValue(throwError('error'));
    component.setVisible(true);
    const activity = {status: 'cancelled', id: '1', ownership: 'currentUser'};
    component.carePlan = {
      id: '123',
      version: 456,
      activities: [ activity ]
    } as any as CarePlan;

    component.editableActivities = []; // not allowed to change any activity
    component.selectedActivities = [true]; // selected (although this would be impossible in gui)

    expect(activity.status).toBe('cancelled');
    component.submit();
    expect(activity.status).toBe('cancelled'); // would be 'expected' if allowed to change
  });

});
