import {Component, OnDestroy, OnInit, ViewChild} from '@angular/core';
import {ModalComponent} from '../../modals/modal/modal.component';
import {FormBuilder, FormGroup, FormGroupDirective, Validators} from '@angular/forms';
import {Activity, CarePlan, Note} from '@nspop/gm-web-facade-api';
import {copy} from '../../../utils/copy';
import {MatInput} from '@angular/material/input';
import {ActivitiesService} from '../activities.service';
import {calculateWeekNumberFromPregnancyStart} from '../../../utils/dates';
import {CarePlanService} from '../../api/care-plan.service';

@Component({
  selector: 'app-note-modal',
  templateUrl: './note-modal.component.html',
  styleUrls: ['./note-modal.component.scss']
})
export class NoteModalComponent implements OnInit, OnDestroy {

  @ViewChild('modal') public modalComponent: ModalComponent | undefined;
  @ViewChild('formDirective') public formDirective: FormGroupDirective | undefined;
  @ViewChild('title') public title: MatInput | undefined;

  public form: FormGroup | undefined;

  public carePlan: CarePlan | undefined;

  public activity: Activity | undefined;
  public isEditMode: boolean | undefined;
  public isDirty: boolean = false;

  constructor(
    private fb: FormBuilder,
    public carePlanService: CarePlanService,
    public activitiesService: ActivitiesService,
    ) { }

  ngOnInit(): void {
    this.form = this.fb.group({
      title: ['', {disabled: true}],
      content: ['', Validators.required],
    });
  }

  ngOnDestroy(): void {
    this.form = undefined;
  }

  public submit(form: FormGroup | undefined, formDirective: FormGroupDirective | undefined): void {
    if (!this.activity) { return; }
    if (!this.carePlan || !this.carePlan.activities) { return; }

    if (form?.valid) {
      const activityIndex = this.carePlan.activities.findIndex((a) =>
        a.id === this.activity?.id && a.id !== undefined);

      if (activityIndex === -1) { return console.error('Could not find activity with id: ' + this.activity.id); }

      const oldActivity = copy(this.carePlan.activities[activityIndex]); // Take copy for reverting purposes

      const note = this.getNoteFromForm();
      note.documentUniqueId = this.activity.note?.documentUniqueId;
      note.version = this.activity.note?.version;
      this.carePlan.activities[activityIndex].note = note;

      this.submitCarePlan(activityIndex, oldActivity, formDirective);
    } else {
      form?.markAllAsTouched();
    }
  }

  private submitCarePlan(activityIndex: number, oldActivity: Activity, formDirective?: FormGroupDirective): void {
    if (!this.carePlan) { return; }
    this.carePlanService.update(this.carePlan, this.carePlan.id + '', this.carePlan.version + '').subscribe(
      () => {
        formDirective?.resetForm();
        this.setVisible(false);
        this.carePlanService.refresh();
      },
      () => {
        if (!this.carePlan?.activities) { return; }
        this.activity = oldActivity;
        this.carePlan.activities[activityIndex] = oldActivity;
      });
  }

  setActivity(activity: Activity | undefined): void {
    this.activity = copy(activity);
    this.isEditMode = !!activity?.note;
  }

  setVisible(isVisible: boolean): void {
    this.carePlan = this.activitiesService.carePlan;
    this.isDirty = false;
    if (this.modalComponent) {
      this.modalComponent.setVisible(isVisible);
      this.formDirective?.resetForm();
      this.setFormValues();
    }
  }

  setFormValues(): void {
    let title = this.activity?.title;
    const from = calculateWeekNumberFromPregnancyStart(this.activitiesService.dataCard, this.activity?.plannedTime?.from);
    const to = calculateWeekNumberFromPregnancyStart(this.activitiesService.dataCard, this.activity?.plannedTime?.to);
    title += from && to
      ? ` (${from} - ${to})`
      : '';

    this.form = this.fb.group({
      title: [title || ''],
      content: [
        {
          value: this.activity?.note?.content || '',
          disabled: this.activity && this.activity.ownership === 'otherUser'
        },
        Validators.required
      ],
    });
    this.form.controls.title.disable();
  }

  isVisible(): boolean {
    this.form?.markAsUntouched();
    return !!this.modalComponent?.isModalVisible;
  }

  getNoteFromForm(): Note {
    const {content} = this.form?.value;
    return {
      content,
      dirty: this.isDirty,
    } as Note;
  }

  formatDate(): string {
    const date = this.activity?.note?.date;
    return date
      ? `${this.padZero(date.getDate())}-${this.padZero(date.getMonth())}-${date.getFullYear()}
       Kl. ${this.padZero(date.getHours())}:${this.padZero(date.getMinutes())}`
      : '';
  }

  private padZero(n: number): string {
    return n < 10 ? `0${n}` : `${n}`;
  }

  public onNoteContentChange(): void {
    this.isDirty = true;
  }
}
