import {Injectable} from '@angular/core';
import {HttpClient, HttpErrorResponse} from '@angular/common/http';
import {Observable, throwError} from 'rxjs';
import {catchError} from 'rxjs/operators';
import {AppError, BadInputError, PageNotFoundError} from '../services/error.service';
import {apiBase} from '../../utils/URL_STRING_LITERALS';

@Injectable({
  providedIn: 'root'
})
export class ApiService {

  constructor(protected url: string,
              private httpClient: HttpClient) {
    this.url = apiBase + url;
  }

  public create(resource: any): Observable<any> {
    return this.httpClient.post(this.url, resource).pipe(
      catchError(this.handleCreateError)
    );
  }

  public getAll(): Observable<any> {
    return this.httpClient.get(this.url).pipe(
      catchError(this.handleGetAllError)
    );
  }

  public get(id: string): Observable<any> {
    return this.httpClient.get(`${this.url}/${id}`).pipe(
      catchError(this.handleGetError)
    );
  }

  public update(resource: any, id: string, version?: string): Observable<any> {
    return this.httpClient.put(version ? `${this.url}/${id}/${version}` : `${this.url}/${id}`, resource).pipe(
      catchError(this.handleUpdateError)
    );
  }

  public delete(id: string): Observable<any> {
    return this.httpClient.delete(`${this.url}/${id}`).pipe(
      catchError(this.handleDeleteError)
    );
  }

  protected extendUrlAndCall(urlExtension: string, callback: () => Observable<any>): Observable<any> {
    const oldUrl = this.url;
    this.url += urlExtension;

    const obs = callback();

    this.url = oldUrl;

    return obs;
  }

  private handleError(error: HttpErrorResponse): Observable<never> {
    if (error.status === 400) {
      return throwError(new BadInputError(error));
    }

    if (error.status === 404) {
      return throwError(new PageNotFoundError(error));
    }

    return throwError(new AppError(error));
  }

  // Errors for overriding (to throw more specific errors):
  protected handleCreateError = (error: HttpErrorResponse): Observable<never> => {
    return this.handleError(error);
  }

  protected handleGetAllError = (error: HttpErrorResponse): Observable<never> => {
    return this.handleError(error);
  }

  protected handleGetError = (error: HttpErrorResponse): Observable<never> => {
    return this.handleError(error);
  }

  protected handleUpdateError = (error: HttpErrorResponse): Observable<never> => {
    return this.handleError(error);
  }

  protected handleDeleteError(error: HttpErrorResponse): Observable<never> {
    return this.handleError(error);
  }
}
