import { Injectable } from '@angular/core';
import {ApiService} from './api.service';
import {HttpClient} from '@angular/common/http';
import {BehaviorSubject, Observable} from 'rxjs';
import {
  SaveUltrasoundObservationsRequest,
  ObservationTable,
  SaveRegularObservationsRequest,
  ScanningObservationTable,
} from '@nspop/gm-web-facade-api';

@Injectable({
  providedIn: 'root'
})
export class ObservationService extends ApiService {

  constructor(httpClient: HttpClient) {
    super('/observation', httpClient);
  }

  // Inner subjects - for manually publishing updates to subscribers
  private innerUltrasoundObservationTable: BehaviorSubject<ScanningObservationTable | undefined>
    = new BehaviorSubject<ScanningObservationTable | undefined>(undefined);
  private innerClinicalObservationTable: BehaviorSubject<ObservationTable | undefined>
    = new BehaviorSubject<ObservationTable | undefined>(undefined);

  // Observables (from inner subjects) - subscribe to these to get updated info!
  public ultrasoundObservationTable$: Observable<ScanningObservationTable | undefined>
    = this.innerUltrasoundObservationTable.asObservable();
  public clinicalObservationTable$: Observable<ObservationTable | undefined>
    = this.innerClinicalObservationTable.asObservable();

  // Refresh methods - these are the ones that actually call the server and manually publishes new data
  public refresh(): void {
    this.refreshUltrasoundObservationTable();
    this.refreshClinicalObservationTable();
  }

  public refreshUltrasoundObservationTable(): void {
    this.getUltrasoundObservationTable().subscribe((table: ScanningObservationTable) => {
      this.innerUltrasoundObservationTable.next(table);
    });
  }

  public refreshClinicalObservationTable(): void {
    this.getClinicalObservationTable().subscribe((table: ObservationTable) => {
      this.innerClinicalObservationTable.next(table);
    });
  }

  // Overriding api.service methods
  createUltrasoundObservations(observations: SaveUltrasoundObservationsRequest): Observable<void> {
    return this.extendUrlAndCall('/scan', () => super.create(observations));
  }

  createRegularObservations(observations: SaveRegularObservationsRequest): Observable<void> {
    return this.extendUrlAndCall('/regular', () => super.create(observations));
  }

  getUltrasoundObservationTable(): Observable<ScanningObservationTable> {
    return this.extendUrlAndCall('/table/scan', () => super.getAll());
  }

  getClinicalObservationTable(): Observable<ObservationTable> {
    return this.extendUrlAndCall('/table/clinical', () => super.getAll());
  }

  updateUltrasoundObservations(observations: SaveUltrasoundObservationsRequest, id: string, version: string): Observable<void> {
    return this.extendUrlAndCall('/scan', () => super.update(observations, id, version));
  }

  updateRegularObservations(observations: SaveRegularObservationsRequest, id: string, version: string): Observable<void> {
    return this.extendUrlAndCall('/regular', () => super.update(observations, id, version));
  }
}
