import {Directive, HostListener, Injectable} from '@angular/core';
import {CanDeactivate, RouterOutlet} from '@angular/router';
import {NgForm} from '@angular/forms';
import {Observable} from 'rxjs';
import {DialogService} from './modals/dialog/dialog.service';
import {DialogComponent} from './modals/dialog/dialog.component';
import {environment} from '../environments/environment';


export interface ConfirmIfDirty {
  isDirty(): boolean;
}

export abstract class FormComponent implements ConfirmIfDirty {
  abstract get form(): NgForm | undefined;

  isDirty(): boolean {
    if (this.form) {
      return !this.form.submitted && !!this.form.dirty;
    }
    return false;
  }
}

// tslint:disable-next-line:directive-selector
@Directive({ selector: 'router-outlet' })
export class RouterOutletDirtyWatchDirective {
  private components: ConfirmIfDirty[] = [];

  constructor(routerOutlet: RouterOutlet) {
    routerOutlet.activateEvents.subscribe((component: any) => {
      if (component) {
        this.components.push(component);
      }
    });
    routerOutlet.deactivateEvents.subscribe( (component: any) => {
      const idx = this.components.indexOf(component);
      if (idx > -1) {
        this.components.splice(idx, 1);
      }
    });
  }

  @HostListener('window:beforeunload', ['$event'])
  public canDeactivate($event: any): boolean {
    for (const component of this.components) {
      if (typeof component.isDirty === 'function' && component.isDirty() && environment.production){
          // Prevent window close/reload if form is dirty... we don't want this when developing since it interferes with live reload
          $event.returnValue = false;
          return false;
      }
    }
    return true;
  }
}

@Injectable({ providedIn: 'root' })
export class DirtyGuard implements CanDeactivate<any> {
  constructor(private dialogService: DialogService) { }

  canDeactivate(component: any): boolean | Promise<boolean> | Observable<boolean> {
    if (component && typeof component.isDirty === 'function' && component.isDirty()){
      return this.dialogService.show(DialogComponent, {data: {
          header: 'Du har foretaget ændringer, som ikke er gemte',
          simpleContent: 'Vælg om du ønsker at fortsætte. Ændringer, som ikke er gemt vil gå tabt',
          noButtonText: 'Fortryd',
          okButtonText: 'Fortsæt'
      }, width: '440'});
    }
    return true;
  }
}

