import {ComponentFixture, fakeAsync, getTestBed, inject, TestBed} from '@angular/core/testing';
import { CprSearchFabComponent } from './cpr-search-fab.component';
import {MatIconModule} from '@angular/material/icon';
import {FormsModule} from '@angular/forms';
import {CprSearchEventService} from '../services/cpr-search-event.service';
import {Subject} from 'rxjs';
import {MatButton} from '@angular/material/button';
import {WindowService} from '../services/window.service';
import {DocumentService} from '../services/document.service';

describe('CprSearchFabComponent', () => {
  let component: CprSearchFabComponent;
  let fixture: ComponentFixture<CprSearchFabComponent>;
  let buttonElement: HTMLButtonElement;
  let cprSearchFocusedSubjectStub: Subject<boolean>;
  let cprSearchEventServiceStub: CprSearchEventService;

  let xStub: number;
  let yStub: number;
  let pageXOffsetStub: number | undefined;
  let compatModeStub: string;
  let windowServiceStub: WindowService;

  let cprSearchFabStub: MatButton;
  let styleStub: CSSStyleDeclaration;

  beforeEach(async () => {
    cprSearchFocusedSubjectStub = new Subject<boolean>();

    cprSearchEventServiceStub = {
      cprSearchFocused$: cprSearchFocusedSubjectStub.asObservable(),
      emitCprSearchFabClickEvent: () => {
      },
    } as any;

    yStub = 71;
    xStub = 0;

    pageXOffsetStub = xStub;

    compatModeStub = '';

    windowServiceStub = {
      nativeWindow: () => ({
        pageXOffset: pageXOffsetStub,
        pageYOffset: yStub,
      } as Window),
      getXYScroll: () => {},
    } as any;

    styleStub = {
      top: '69px',
      left: '69px',
      position: 'auto',
    } as any;

    cprSearchFabStub = {
      _elementRef: {
        nativeElement: {
          style: styleStub,
        } as any,
      } as any,
    } as any;

    await TestBed.configureTestingModule({
      declarations: [CprSearchFabComponent],
      imports: [MatIconModule, FormsModule],
      providers: [
        {provide: CprSearchEventService, useValue: cprSearchEventServiceStub},
        {provide: WindowService, useValue: windowServiceStub},
      ]
    })
      .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(CprSearchFabComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();

    buttonElement = fixture.elementRef.nativeElement.querySelector('button');
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('is not disabled by default', () => {
    expect(component.cprSearchFab?.disabled).toBeFalsy();
  });

  describe('when windowService.getXYScroll returns y < 71', () => {
    beforeEach(async () => {
      component.cprSearchFab = cprSearchFabStub;
      spyOn(windowServiceStub, 'getXYScroll').and.returnValue({x: 0, y: 71});
    });

    it('sets style.position = relative and style.top = 0px', () => {
      component.allowYScrollFrame();
      expect(styleStub.position).toBe('relative');
      expect(styleStub.top).toBe('0px');
    });
  });

  describe('when windowService.getXYScroll returns y < 71', () => {
    beforeEach(async () => {
      component.cprSearchFab = cprSearchFabStub;
      spyOn(windowServiceStub, 'getXYScroll').and.returnValue({x: 0, y: 72});
    });

    it('sets style.position = fixed and style.top = 10px', () => {
      component.allowYScrollFrame();
      expect(styleStub.position).toBe('fixed');
      expect(styleStub.top).toBe('10px');
    });
  });


  describe('when it is clicked', () => {
    beforeEach((done) => {
      fixture.whenStable().then(() => {
        spyOn(cprSearchEventServiceStub, 'emitCprSearchFabClickEvent');
        spyOn(component, 'onCprSearchFabClick').and.callThrough();
        buttonElement.dispatchEvent(new Event('click'));
        fixture.detectChanges();
        done();
      });
    });

    it('calls onCprSearchFabClick', () => {
      expect(component.onCprSearchFabClick).toHaveBeenCalledTimes(1);
    });

    it('emits click event', () => {
      expect(cprSearchEventServiceStub.emitCprSearchFabClickEvent).toHaveBeenCalledTimes(1);
    });
  });

  describe('when cprSearchFocus event with value "true" is emitted', () => {
    beforeEach(() => {
      cprSearchFocusedSubjectStub.next(true);
    });

    it('disables the fab', fakeAsync(() => {
      expect(component.cprSearchFab?.disabled).toBe(true);
    }));

    describe('and then cprSearchFocus event with value "false" is emitted', () => {
      beforeEach(() => {
        cprSearchFocusedSubjectStub.next(false);
      });

      it('enables the fab', fakeAsync(() => {
        expect(component.cprSearchFab?.disabled).toBe(false);
      }));
    });
  });
});
