import {Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges, ViewChild} from '@angular/core';
import {ModalComponent} from '../../modals/modal/modal.component';
import {
  Activity,
  SaveRegularObservationsRequest,
  SaveUltrasoundObservationsRequest,
  DataCard, Observation, ObservationType
} from '@nspop/gm-web-facade-api';
import {FormComponent} from '../../confirm-navigation-if-dirty.framework';
import {NgForm} from '@angular/forms';
import {copy} from '../../../utils/copy';
import {merge, Observable, of} from 'rxjs';
import {finalize} from 'rxjs/operators';
import {DataCardWrapper} from '../../model-wrappers/dataCard.wrapper';
import {ObservationService} from '../../api/observation.service';
import {DataCardService} from '../../api/data-card.service';
import {CarePlanService} from '../../api/care-plan.service';


type ObservationFormType = 'regular' | 'ultrasound';
@Component({
  selector: 'app-edit-measurements-modal',
  templateUrl: './edit-measurements-modal.component.html',
  styleUrls: ['./edit-measurements-modal.component.scss']
})
export class EditMeasurementsModalComponent extends FormComponent implements OnChanges, OnInit {
  public note = '';
  public isSaving = false;
  public dataCard?: DataCardWrapper;
  public activityId: string | undefined;
  public observationDataRegular?: SaveRegularObservationsRequest | undefined;
  public observationDataUltrasound?: SaveUltrasoundObservationsRequest | undefined;
  public editMode: boolean = false;
  public readOnlyMode: boolean = false;

  @ViewChild('modal') public modalComponent: ModalComponent | undefined;
  @ViewChild('form') public form: NgForm | undefined;

  @Input() public isOpen = false;
  @Input() public observationType: ObservationFormType = 'regular';
  @Input() public allowTypeChange = false;
  @Input() public activity: Activity | undefined = undefined;
  @Input() public allowActivityChange = true;

  @Output() public modalClosed: EventEmitter<void> = new EventEmitter();
  @Output() public save: EventEmitter<void> = new EventEmitter();

  private dataCardId = '';
  private dataCardVersion = '';

  private observationId: string = '';
  private observationVersion: string = '';

  constructor(
    private carePlanService: CarePlanService,
    private dataCardService: DataCardService,
    private observationService: ObservationService,
  ) {
    super();
  }

  ngOnInit(): void {
    this.dataCardService.activeDataCard$.subscribe((dataCard?: DataCard) => {
      if (dataCard) {
        this.dataCardId = dataCard.documentUniqueId || '';
        this.dataCardVersion = dataCard.version || '';
        this.dataCard = new DataCardWrapper(copy(dataCard));
      }
    });

    this.form?.valueChanges?.subscribe((change) => {
      console.log('Form changed!:', change);
    });
  }

  ngOnChanges(changes: SimpleChanges): void {
    console.log('Changes', changes);
    if (changes.isOpen) {
      if (this.isOpen) {
        this.initObservationData();
        this.modalComponent?.setVisible(true);
      } else {
        this.handleClose();
      }
    }
    if (changes.activity) {
      if (this.activity) {
        console.log('activity change', this.activity);
        this.activityId = this.activity?.id;
        this.editMode = !!this.activity?.observations && this.activity.observations.length > 0;
        this.readOnlyMode = this.activity.ownership === 'otherUser';
        this.setInitialObservationValues();
      }
    }
  }

  public handleClose(): void {
    if (!this.isOpen) { return; }

    this.modalComponent?.setVisible(false);

    if (this.modalComponent?.isModalVisible) {return; }
    this.modalClosed.emit();

    this.clearObservationData();
  }

  public validateActivityId(): boolean {
    const activitySelector = document.getElementById('activitySelector');
    if (!this.activityId && this.modalComponent?.isModalVisible) {
      activitySelector?.classList.add('ng-invalid');
      return false;
    }
    activitySelector?.classList.remove('ng-invalid');
    return true;
  }

  public doSave(): void {
    if (!this.validateActivityId()) {
      const activitySelector = document.getElementById('activitySelector');
      activitySelector?.scrollIntoView({behavior: 'smooth', block: 'center'});
      return;
    }
    this.setActivityId(this.activityId);
    this.isSaving = true;
    const save = (this.observationType === 'ultrasound') ? this.saveUltrasound() : this.saveRegular();
    save// TODO: create note too (Eller hvad? Skal vi se om der allerede er en på aktiviteten og hvis ja redigere den?)
      .pipe(finalize(() => this.isSaving = false))
    .subscribe(() => {
      this.handleClose();
      this.dataCardService.refresh(); // updating dataCard => update episodeOfCare => update carePlan & observations (see app.component)
      this.save.emit();
    });
  }

  private saveUltrasound(): Observable<void> {
    if (this.dataCard) {
      console.log('Saving ultraSound!', this.observationDataUltrasound);
      const saveUltraSoundObservable = this.editMode
        ? this.observationService.updateUltrasoundObservations(
          this.observationDataUltrasound as SaveUltrasoundObservationsRequest,
          this.observationId,
          this.observationVersion)
        : this.observationService.createUltrasoundObservations(this.observationDataUltrasound as SaveUltrasoundObservationsRequest);

      return merge(
        this.dataCardService.update(this.dataCard.asDataCard, this.dataCardId, this.dataCardVersion),
        saveUltraSoundObservable
      ) as unknown as Observable<void>;
    }
    return of();
  }

  private saveRegular(): Observable<void> {
    console.log('Saving regular!', this.observationDataRegular);
    // return of();
    return this.editMode
      ? this.observationService.updateRegularObservations(
        this.observationDataRegular as SaveRegularObservationsRequest,
        this.observationId,
        this.observationVersion)
      : this.observationService.createRegularObservations(this.observationDataRegular as SaveRegularObservationsRequest);
  }

  private setActivityId(activityId?: string): void {
    if (!this.observationDataRegular || !this.observationDataUltrasound) { return; }
    this.observationDataRegular.activityId = activityId;
    this.observationDataUltrasound.activityId = activityId;
  }

  private initObservationData(): void {
    this.observationDataRegular = {
      fetuses: [...new Array(this.dataCard?.fetusCount || 1)].map((x, index) => ({id: `Foster ${index + 1}`})),
      bloodPressure: undefined,
    };

    this.observationDataUltrasound = {
      fetuses: [...new Array(this.dataCard?.fetusCount || 1)].map((x, index) => ({id: `Foster ${index + 1}`})),
    };

    this.observationDataRegular = copy(this.observationDataRegular);
    this.observationDataUltrasound = copy(this.observationDataUltrasound);

    this.form?.form.markAsPristine();
  }

  public isObservationDataInitialized(): boolean {
    return !!this.observationDataRegular && !!this.observationDataUltrasound;
  }

  private clearObservationData(): void {
    this.observationDataRegular = undefined;
    this.observationDataUltrasound = undefined;
  }

  private setInitialObservationValues(): void {
    this.observationId = '';
    this.observationVersion = '';
    this.activity?.observations?.forEach((observation: Observation) => {
      if (!this.observationDataRegular || !this.observationDataUltrasound) { return; }
      switch (observation.type) {
        case ObservationType.Weight:
          this.setModelWeight(observation);
          break;
        case ObservationType.BloodPressure:
          this.setModelBloodPressure(observation);
          break;
        case ObservationType.Urine:
          this.setModelUrine(observation);
          break;
        case ObservationType.Edema:
          this.setModelEdema(observation);
          break;
        case ObservationType.PubisMeasurements:
          this.setModelPubisMeasurements(observation);
          break;
        case ObservationType.FetusActivity:
          this.setModelFetusActivity(observation);
          break;
        case ObservationType.FetusWeightEstimate:
          this.setModelFetusWeightEstimate(observation);
          break;
        case ObservationType.FetusPresentation:
          this.setModelFetusPresentation(observation);
          break;
        case ObservationType.Cervix:
          this.setModelCervix(observation);
          break;
        case ObservationType.Dvp:
          this.setModelDvp(observation);
          break;
        case ObservationType.UlWeight:
          this.setModelUlWeight(observation);
          break;
        case ObservationType.WeightDeviation:
          this.setModelWeightDeviation(observation);
          break;
        case ObservationType.AmnioticFluid:
          this.setModelAmnioticFluid(observation);
          break;
        case ObservationType.Flow:
          this.setModelFlow(observation);
          break;
      }
      this.observationId = observation.id ?? '';
      this.observationVersion = observation.version ? observation.version + '' : '';
    });
    this.observationDataRegular = copy(this.observationDataRegular);
    this.observationDataUltrasound = copy(this.observationDataUltrasound);
  }

  private setModelWeight(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    if (!this.observationDataRegular) { return; }

    this.observationDataRegular.weight = +observation.values[0]?.value;
  }

  private setModelBloodPressure(observation: Observation): void {
    if (observation.values.length !== 2) { return; }
    if (!this.observationDataRegular?.bloodPressure) { return; }

    this.observationDataRegular.bloodPressure = {
      systolic: +observation.values[0]?.value,
      diastolic: +observation.values[1]?.value,
    };
  }

  private setModelUrine(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    if (!this.observationDataRegular) { return; }

    this.observationDataRegular.urine = observation.values[0]?.value;
  }

  private setModelEdema(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    if (!this.observationDataRegular) { return; }

    this.observationDataRegular.edema = observation.values[0]?.value;
  }

  private setModelPubisMeasurements(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    if (!this.observationDataRegular) { return; }

    this.observationDataRegular.pubismeasurement = +observation.values[0]?.value;
  }

  private getFetusNumberFromObservation(observation: Observation): number {
    if (!observation.fetus) {return -1; }

    return +observation.fetus.split(' ')[1]; // e.g. 'Foster 1' = 1
  }

  private setModelFetusActivity(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    const fetusNumber = this.getFetusNumberFromObservation(observation);
    if (!this.observationDataRegular?.fetuses || this.observationDataRegular.fetuses.length < fetusNumber) { return; }

    const fetusActivity = observation.values[0]?.value.split(', '); // e.g. '+liv, +hjertelyd' = ['+liv', '+hjertelyd']
    this.observationDataRegular.fetuses[fetusNumber - 1].fetusActivity = fetusActivity;
  }

  private setModelFetusWeightEstimate(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    const fetusNumber = this.getFetusNumberFromObservation(observation);
    if (!this.observationDataRegular?.fetuses || this.observationDataRegular.fetuses.length < fetusNumber) { return; }

    this.observationDataRegular.fetuses[fetusNumber - 1].fetusWeightEstimate = +observation.values[0]?.value;
  }

  private setModelFetusPresentation(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    const fetusNumber = this.getFetusNumberFromObservation(observation);

    // Regular observations
    if (!this.observationDataRegular?.fetuses || this.observationDataRegular.fetuses.length < fetusNumber) { return; }
    this.observationDataRegular.fetuses[fetusNumber - 1].fetusPresentation = observation.values[0]?.value;

    // Ultrasound observations
    if (!this.observationDataUltrasound?.fetuses || this.observationDataUltrasound.fetuses.length < fetusNumber) { return; }
    this.observationDataUltrasound.fetuses[fetusNumber - 1].fetusPresentation = observation.values[0]?.value;
  }

  private setModelCervix(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    if (!this.observationDataUltrasound) { return; }

    this.observationDataUltrasound.cervix = +observation.values[0]?.value;
  }

  private setModelDvp(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    if (!this.observationDataUltrasound) { return; }

    this.observationDataUltrasound.dvp = +observation.values[0]?.value;
  }

  private setModelUlWeight(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    const fetusNumber = this.getFetusNumberFromObservation(observation);
    if (!this.observationDataUltrasound?.fetuses || this.observationDataUltrasound.fetuses.length < fetusNumber) { return; }

    this.observationDataUltrasound.fetuses[fetusNumber - 1].ulWeight = +observation.values[0]?.value;
  }

  private setModelWeightDeviation(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    const fetusNumber = this.getFetusNumberFromObservation(observation);
    if (!this.observationDataUltrasound?.fetuses || this.observationDataUltrasound.fetuses.length < fetusNumber) { return; }

    this.observationDataUltrasound.fetuses[fetusNumber - 1].weightDeviation = +observation.values[0]?.value;
  }

  private setModelAmnioticFluid(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    if (!this.observationDataUltrasound) { return; }

    this.observationDataUltrasound.amnioticFluid = observation.values[0]?.value;
  }

  private setModelFlow(observation: Observation): void {
    if (observation.values.length !== 1) { return; }
    if (!this.observationDataUltrasound) { return; }

    this.observationDataUltrasound.flow = observation.values[0]?.value;
  }

}
