import {ComponentFixture, TestBed} from '@angular/core/testing';

import {BloodPressureInputComponent} from './blood-pressure-input.component';
import {AngularMaterialModule} from '../../../../modules/angular-material.module';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {FormsModule, NgForm} from '@angular/forms';

describe('BloodPressureInputComponent', () => {
  let component: BloodPressureInputComponent;
  let fixture: ComponentFixture<BloodPressureInputComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ BloodPressureInputComponent ],
      imports: [AngularMaterialModule, NoopAnimationsModule, FormsModule],
      providers: [
        {provide: NgForm, useValue: new NgForm([], [])}
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(BloodPressureInputComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('changes focus to diastolic textbox when "/" is pressed', () => {
    const diastolic = component.diastolicInput?.nativeElement as HTMLInputElement;
    const systolic = component.systolicInput?.nativeElement as HTMLInputElement;
    const spy = spyOn(diastolic as HTMLOrSVGElement, 'focus');
    emulateKeystroke(systolic, '1');
    emulateKeystroke(systolic, '0');
    emulateKeystroke(systolic, '/');

    expect(spy).toHaveBeenCalled();
  });

  it('changes focus to diastolic textbox when "-" is pressed', () => {
    const diastolic = component.diastolicInput?.nativeElement as HTMLInputElement;
    const systolic = component.systolicInput?.nativeElement as HTMLInputElement;
    const spy = spyOn(diastolic as HTMLOrSVGElement, 'focus');
    emulateKeystroke(systolic, '1');
    emulateKeystroke(systolic, '0');
    emulateKeystroke(systolic, '-');

    expect(spy).toHaveBeenCalled();
  });

  it('changes focus to diastolic textbox when ":" is pressed', () => {
    const diastolic = component.diastolicInput?.nativeElement as HTMLInputElement;
    const systolic = component.systolicInput?.nativeElement as HTMLInputElement;
    const spy = spyOn(diastolic as HTMLOrSVGElement, 'focus');
    emulateKeystroke(systolic, '1');
    emulateKeystroke(systolic, '0');
    emulateKeystroke(systolic, ':');

    expect(spy).toHaveBeenCalled();
  });

  it('does not change focus if no keystroke triggers it', () => {
    const diastolic = component.diastolicInput?.nativeElement as HTMLInputElement;
    const systolic = component.systolicInput?.nativeElement as HTMLInputElement;
    const spy = spyOn(diastolic as HTMLOrSVGElement, 'focus');
    emulateKeystroke(systolic, '1');
    emulateKeystroke(systolic, '0');
    emulateKeystroke(systolic, '0');

    expect(spy).not.toHaveBeenCalled();
  });

  it('returns undefined if both systolic and diastolic is empty', () => {
    const onChange = jasmine.createSpy('onChange');
    component.writeValue({systolic: 5, diastolic: 4});
    component.registerOnChange(onChange);
    component.handleFocus();
    component.writeValue({systolic: 5, diastolic: undefined});
    expect(onChange).toHaveBeenCalledWith({systolic: 5, diastolic: undefined});
    component.writeValue({systolic: undefined, diastolic: undefined});
    expect(onChange).toHaveBeenCalledWith(undefined);

  });

  it('only registers changes if model has structural changes', () => {
    const onChange = jasmine.createSpy('onChange');
    component.writeValue({systolic: 5, diastolic: 4});
    component.registerOnChange(onChange);
    component.writeValue({systolic: 5, diastolic: 4});
    expect(onChange).toHaveBeenCalledTimes(0);
    component.handleFocus();
    component.writeValue({systolic: 4, diastolic: 4});
    expect(onChange).toHaveBeenCalledTimes(1);
  });

  it('updates inner value when ngModel is updated on the outside', () => {
    component.handleFocus();
    component.writeValue({systolic: 5, diastolic: 4});
    expect(component.systolic).toEqual(5);
    expect(component.diastolic).toEqual(4);
  });

  it('does not register a change on the first value set', () => {
    // This is to avoid issues where values initialized from an async call will mark the form as dirty
    const mock = {
      onChange: (whatever: any) => undefined
    };
    const onChange = spyOn(mock, 'onChange');
    component.registerOnChange(onChange);
    component.writeValue({systolic: 5, diastolic: 4});
    expect(onChange).not.toHaveBeenCalled();
    component.handleFocus();
    component.writeValue({systolic: 5, diastolic: 5});
    expect(onChange).toHaveBeenCalled();
  });
});

function emulateKeystroke(input: HTMLInputElement, char: string): void {
  input.value += char;
  input.dispatchEvent(new KeyboardEvent('keydown', {key: char}));
}
