import {ComponentFixture, TestBed} from '@angular/core/testing';
import {AbortionBundle, BirthBundle, PreviousPregnanciesComponent} from './previous-pregnancies.component';
import {DataCard, PreviousAbortionsSectionAllOfAbortions, PreviousBirthsSectionAllOfBirths} from '@nspop/gm-web-facade-api';
import {JsonTransformerUtil} from '../../../utils/json-transformer.util';
import {FormsModule, NgForm} from '@angular/forms';

describe('PreviousPregnanciesComponent', () => {
  let component: PreviousPregnanciesComponent;
  let fixture: ComponentFixture<PreviousPregnanciesComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ PreviousPregnanciesComponent ],
      imports: [FormsModule],
      providers: [
        {provide: NgForm, useValue: new NgForm([], [])},
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PreviousPregnanciesComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('only registers changes if model has structural changes', () => {
    const dataCard1 = {
      previousAbortionsSection: {
        abortions: []
      },
      previousBirthsSection: {
        births: []
      }
    } as any as DataCard;
    const dataCard2 = JsonTransformerUtil.transformDates(JSON.parse(JSON.stringify(dataCard1))) as any as DataCard;
    // tslint:disable-next-line:no-non-null-assertion It's not null because it's a copy of dataCard1 which has the property set...
    dataCard2.previousAbortionsSection!.abortions!.push({
      date: undefined,
      type: 'induced',
      gestationLength: {
        value: 6,
        unit: 'w'
      },
      indication: 'whatever'
    });
    const onChange = jasmine.createSpy('onChange');
    component.writeValue(dataCard1 );
    component.registerOnChange(onChange);
    expect(component.abortions.length).toBe(0);
    component.writeValue(JsonTransformerUtil.transformDates(JSON.parse(JSON.stringify(dataCard1))) as any as DataCard);
    expect(onChange).toHaveBeenCalledTimes(0);
    component.writeValue(dataCard2 as any as DataCard);
    expect(onChange).toHaveBeenCalledTimes(1);
    expect(component.abortions.length).toBe(1);
  });

  it('adds an empty birth when addBirth is called', () => {
    const dataCard = {
      previousAbortionsSection: {
        abortions: []
      },
      previousBirthsSection: {
        births: []
      }
    } as any as DataCard;
    component.writeValue(dataCard);
    expect(component.births.length).toBe(0);
    component.addBirth();
    expect(component.births.length).toBe(1);
  });

  it('removes a specifc birth when removeBirth is called', () => {
    const dataCard = {
      previousAbortionsSection: {
        abortions: []
      },
      previousBirthsSection: {
        births: []
      }
    } as any as DataCard;
    component.writeValue(dataCard);
    component.addBirth();
    component.addBirth();
    component.births[0].birthYear = 2000;
    component.births[1].birthYear = 2010;

    component.removeBirth(component.births[0]);

    expect(component.births.length).toBe(1);
    expect(component.births[0].birthYear).toBe(2010);
    expect(component.value?.previousBirthsSection?.births?.length).toBe(1);
    expect(component.value?.previousBirthsSection?.births?.[0]?.date?.getUTCFullYear()).toBe(2010);
  });

  it('adds an empty abortion when addAbortion is called', () => {
    const dataCard = {
      previousAbortionsSection: {
        abortions: []
      },
      previousBirthsSection: {
        births: []
      }
    } as any as DataCard;
    component.writeValue(dataCard);
    expect(component.abortions.length).toBe(0);
    component.addAbortion();
    expect(component.abortions.length).toBe(1);
  });

  it('removes a specifc abortion when removeAbortion is called', () => {
    const dataCard = {
      previousAbortionsSection: {
        abortions: []
      },
      previousBirthsSection: {
        births: []
      }
    } as any as DataCard;
    component.writeValue(dataCard);
    component.addAbortion();
    component.addAbortion();
    component.abortions[0].year = 2000;
    component.abortions[1].year = 2010;

    component.removeAbortion(component.abortions[0]);
    expect(component.abortions.length).toBe(1);
    expect(component.abortions[0].year).toBe(2010);
    expect(component.value?.previousAbortionsSection?.abortions?.length).toBe(1);
    expect(component.value?.previousAbortionsSection?.abortions?.[0]?.date?.getUTCFullYear()).toBe(2010);
  });

  it('adds an empty fetus to a birth when addFetus is called', () => {
    const dataCard = {
      previousAbortionsSection: {
        abortions: []
      },
      previousBirthsSection: {
        births: []
      }
    } as any as DataCard;
    component.writeValue(dataCard);
    component.addBirth();
    expect(component.births[0]?.birth?.children?.length).toBe(1);
    component.addFetus(component.births[0]);
    expect(component.births[0]?.birth?.children?.length).toBe(2);
  });

  it('removes a specific fetus from a birth when removeFetus is called', () => {
    const dataCard = {
      previousAbortionsSection: {
        abortions: []
      },
      previousBirthsSection: {
        births: []
      }
    } as any as DataCard;
    component.writeValue(dataCard);
    component.addBirth();
    expect(component.births[0].birth.children).toBeTruthy();
    expect(component.births[0].birth.children?.length).toBe(1);
    component.addFetus(component.births[0]);
    expect(component.births[0]?.birth?.children?.length).toBe(2);
    // @ts-ignore
    component.births[0].birth.children[0].sex = 'female';
    // @ts-ignore
    component.births[0].birth.children[1].sex = 'male';

    component.removeFetus(component.births[0], 0);
    expect(component.births[0].birth.children?.length).toBe(1);
    // @ts-ignore
    expect(component.births[0].birth?.children[0].sex).toBe('male');
  });


});

describe('AbortionBundle', () => {
  let abortion: PreviousAbortionsSectionAllOfAbortions;
  beforeEach(() => {
    abortion = {
      date: undefined,
      type: 'induced',
      gestationLength: {
        value: 6,
        unit: 'w'
      },
      indication: 'whatever'
    };
  });

  it('gets year from abortion if date is set', () => {
    abortion.date = new Date(Date.parse('12-24-2020 00:00:00'));
    const bundle = new AbortionBundle(abortion, 0);
    expect(bundle.year).toEqual(2020);
  });

  it('returns undefined if abortion date is not set', () => {
    const bundle = new AbortionBundle(abortion, 0);
    expect(bundle.year).toEqual(undefined);
  });

  it('it sets the year-part of the date in the abortion', () => {
    abortion.date = new Date(Date.parse('12-24-2020 00:00:00'));
    const bundle = new AbortionBundle(abortion, 0);
    bundle.year = 2000;

    expect(abortion.date.getUTCFullYear()).toEqual(2000);
  });

  it('it removes date from abortion if year is set to undefined', () => {
    abortion.date = new Date(Date.parse('12-24-2020 00:00:00'));
    const bundle = new AbortionBundle(abortion, 0);
    bundle.year = undefined;

    expect(abortion.date).toBeUndefined();
  });
});

describe('BirthBundle', () => {
  let birth: PreviousBirthsSectionAllOfBirths;
  beforeEach(() => {
    birth = {
      date: undefined,
      gestationLength: {
        value: 200,
        unit: 'd'
      },
      birthPlace: 'some place',
      pregnancySummary: 'text',
      birthSummary: 'more text',
      subjectiveBirthExperience: 'good',
      puerperalSepsis: true,
      children: [{
        sex: 'female',
        birthWeight: {
          value: 3000,
          unit: 'g'
        },
        outcome: 'livebirth',
        childsCurrentHealthStatus: 'even more more text'
      }]
    };
  });

  it('gets year from birth if date is set', () => {
    birth.date = new Date(Date.parse('12-24-2020 00:00:00'));
    const bundle = new BirthBundle(birth, 0);
    expect(bundle.birthYear).toEqual(2020);
  });

  it('returns undefined if birth date is not set', () => {
    const bundle = new BirthBundle(birth, 0);
    expect(bundle.birthYear).toEqual(undefined);
  });

  it('it sets the year-part of the date in the birth', () => {
    birth.date = new Date(Date.parse('12-24-2020 00:00:00'));
    const bundle = new BirthBundle(birth, 0);
    bundle.birthYear = 2000;

    expect(birth.date.getUTCFullYear()).toEqual(2000);
  });

  it('it removes date from birth if year is set to undefined', () => {
    birth.date = new Date(Date.parse('12-24-2020 00:00:00'));
    const bundle = new BirthBundle(birth, 0);
    bundle.birthYear = undefined;

    expect(birth.date).toBeUndefined();
  });

  it('formats gestation age', () => {
    const bundle = new BirthBundle(birth, 0);
    expect(bundle.gestationAge).toEqual('28+4');
  });

  it('returns "0+0" if gestationLength is not set', () => {
    // tslint:disable-next-line:no-non-null-assertion
    (birth.gestationLength!.value as any) = undefined;
    const bundle = new BirthBundle(birth, 0);
    expect(bundle.gestationAge).toEqual('0+0');
  });

  it('it sets the gestation age if correct age is entered', () => {
    const bundle = new BirthBundle(birth, 0);
    bundle.gestationAge = '10+5';

    expect(birth.gestationLength?.value).toEqual(75);
  });

  it('it sets the gestation age if correctable age is entered (e.g. 7+8)', () => {
    const bundle = new BirthBundle(birth, 0);
    bundle.gestationAge = '10+8';

    expect(birth.gestationLength?.value).toEqual(78);
    expect(bundle.gestationAge).toEqual('11+1');
  });

  it('it removes gestationLength if empty string is entered', () => {
    const bundle = new BirthBundle(birth, 0);
    bundle.gestationAge = '';

    expect(birth.gestationLength?.value).toBe(0);
  });

  it('it removes gestationLength if unparsable data is entered is entered', () => {
    const bundle = new BirthBundle(birth, 0);
    bundle.gestationAge = 'What even is this?!';

    expect(birth.gestationLength?.value).toBe(0);
  });
});
