import {Injectable} from '@angular/core';
import {
  HttpErrorResponse,
  HttpEvent,
  HttpHandler,
  HttpHeaders,
  HttpInterceptor,
  HttpRequest,
  HttpResponse
} from '@angular/common/http';
import {Observable, throwError} from 'rxjs';
import {catchError, finalize, map} from 'rxjs/operators';
import {JsonTransformerUtil} from './json-transformer.util';
import {ActiveRequests} from '../app/services/active-requests.service';
import {UserContextHolder} from '../app/services/user-context.service';
import {PatientContextService} from '../app/services/patient-context.service';
import {ErrorService} from '../app/services/error.service';

@Injectable()
export class HttpRequestInterceptor implements HttpInterceptor {

  constructor(
    private activeRequests: ActiveRequests,
    private patientContextService: PatientContextService,
    private userContextHolder: UserContextHolder,
    private errorService: ErrorService,
  ) {}

  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    let headers = this.addHeader(request.headers, 'X-Chosen-Role', this.userContextHolder.currentRole?.authorizationCode);
    headers = this.addHeader(headers, 'X-Organization-Sor', this.userContextHolder.currentOrganization?.sorId);
    headers = this.addHeader(headers, 'X-Institution-Owner-Sor', this.userContextHolder.currentOrganization?.institutionOwnerSorId);
    headers = this.addHeader(headers, 'X-Patient-Identifier', this.patientContextService.getCurrentCpr());
    headers = this.addHeader(headers, 'X-Episode-Of-Care-Identifier', this.patientContextService.getCurrentEpisodeOfCare());
    headers = this.addHeader(headers, 'X-Platform', 'Web');
    headers = this.addHeader(headers, 'X-Break-The-Glass-Reason', this.patientContextService.getCurrentXBreakTheGlassReason());

    const requestWithHeaders = request.clone({headers});

    this.activeRequests._registerBlockingRequest();
    return next.handle(requestWithHeaders).pipe(
      map((event: HttpEvent<any>) => {
        if (event instanceof HttpResponse) {
          const replaced = JsonTransformerUtil.transformDates(event.body);
          return event.clone({body: replaced});
        }
        return event;
      }),
      catchError((error: HttpErrorResponse) => {
        this.errorService.redirectIfRequested(error);
        return throwError(error);
      }),
      finalize(() => {
        this.activeRequests._unregisterBlockingRequest();
      })
    );
  }

  private addHeader(headers: HttpHeaders, header: string, value?: string): HttpHeaders {
    if (value) {
      return (headers || new HttpHeaders()).append(header, value);
    }
    return (headers || new HttpHeaders());
  }
}
